package volgo-git-backend

  1. Overview
  2. Docs

Module Private.Munged_pathSource

Util to parse paths that are written using a munged form by git.

Example of such paths:

1. A simple path:

  "a/simple/path"

2. A pair of paths, with an arrow separator

  "a/simple/path => another/path"

3. (The more involved case). A pair of paths, with common parts

  "a/{simple => not/so/simple}/path"

This module is able to parse all these forms and returned a typed version of it.

Sourceval sexp_of_t : t -> Sexplib0.Sexp.t
Sourceval equal : t -> t -> bool
Sourceval parse_exn : string -> t